"""Tests for API commands."""

import shutil

from pathlib import Path

from expression import Error, Ok, effect
from expression.collections import Map

from fcship.commands.api import (
    ApiContext,
    api,
    create_api,
    create_api_files,
    ensure_api_directories,
    notify_success,
    prepare_api_files,
    validate_api_name,
)
from fcship.utils import FileCreationTracker, FileError


def run_effect(effect_fn):
    """Helper function to run effect functions without returning Result"""
    try:
        result = None
        for step in effect_fn():
            result = step
        return result
    except Exception as e:
        raise e


def cleanup_api_files(name: str) -> None:
    """Clean up API files after test"""
    # Clean up v1 API file
    api_file = Path(f"api/v1/{name}.py")
    if api_file.exists():
        api_file.unlink()
        if not any(api_file.parent.iterdir()):
            api_file.parent.rmdir()

    # Clean up schema file
    schema_file = Path(f"api/schemas/{name}.py")
    if schema_file.exists():
        schema_file.unlink()
        if not any(schema_file.parent.iterdir()):
            schema_file.parent.rmdir()

    # Clean up test file
    test_file = Path(f"tests/api/test_{name}.py")
    if test_file.exists():
        test_file.unlink()
        if not any(test_file.parent.iterdir()):
            test_file.parent.rmdir()

    # Clean up parent directories if empty
    api_dir = Path("api")
    if api_dir.exists() and not any(api_dir.iterdir()):
        api_dir.rmdir()

    tests_dir = Path("tests/api")
    if tests_dir.exists() and not any(tests_dir.iterdir()):
        tests_dir.rmdir()


def cleanup_test_directories():
    """Clean up test directories recursively"""
    for directory in ["api", "tests/api"]:
        path = Path(directory)
        if path.exists():
            shutil.rmtree(path)


def test_create_api_success():
    """Test successful API creation"""

    @effect.result[None, str]()
    def run_test():
        result = yield from create_api("test_endpoint")
        assert result.is_ok()
        assert "Created API endpoint test_endpoint" in result.ok
        # Check if files were created
        api_file = Path("api/v1/test_endpoint.py")
        schema_file = Path("api/schemas/test_endpoint.py")
        test_file = Path("tests/api/test_test_endpoint.py")
        assert api_file.exists()
        assert schema_file.exists()
        assert test_file.exists()

    run_test()
    cleanup_api_files("test_endpoint")


def test_create_api_invalid_name():
    """Test API creation with invalid name"""

    @effect.result[None, str]()
    def test_impl():
        result = yield from create_api("")
        assert result.is_error()
        assert "Invalid API name" in result.error
        yield Ok(None)

    run_effect(test_impl)


def test_api_command_success():
    """Test API command execution success"""
    try:

        @effect.result[None, str]()
        def test_impl():
            result = yield from api("create", "test_api")
            assert result.is_ok()
            assert "Created API endpoint test_api" in result.ok
            # Check if files were created
            api_file = Path("api/v1/test_api.py")
            schema_file = Path("api/schemas/test_api.py")
            test_file = Path("tests/api/test_test_api.py")
            assert api_file.exists()
            assert schema_file.exists()
            assert test_file.exists()
            yield Ok(None)

        run_effect(test_impl)
    finally:
        cleanup_api_files("test_api")


def test_api_command_invalid_operation():
    """Test API command with invalid operation"""

    @effect.result[None, str]()
    def test_impl():
        result = yield from api("invalid", "test_api")
        assert result.is_error()
        assert "Invalid operation" in result.error
        yield Ok(None)

    run_effect(test_impl)


def test_create_api_with_mock(mocker):
    """Test API creation with mocked dependencies"""
    try:

        @effect.result[ApiContext, str]()
        def mock_prepare_files(name: str):
            yield Ok(ApiContext(name=name, files=Map([("test.py", "content")])))

        @effect.result[FileCreationTracker, str]()
        def mock_create_files(files: Map[str, str], base_path: str):
            yield Ok(FileCreationTracker())

        mocker.patch("fcship.commands.api.prepare_api_files", mock_prepare_files)
        mocker.patch("fcship.utils.create_files", mock_create_files)

        @effect.result[None, str]()
        def test_impl():
            result = yield from create_api("test_mock")
            assert result.is_ok()
            assert "Created API endpoint test_mock" in result.ok
            yield Ok(None)

        run_effect(test_impl)
    finally:
        cleanup_api_files("test_mock")


def test_create_api_prepare_error(mocker):
    """Test API creation when prepare files fails"""
    try:

        @effect.result[ApiContext, str]()
        def mock_prepare_files_error(name: str):
            yield Error("Mock prepare error")

        mocker.patch("fcship.commands.api.prepare_api_files", mock_prepare_files_error)

        @effect.result[None, str]()
        def test_impl():
            result = yield from create_api("test_error")
            assert result.is_error()
            assert "Mock prepare error" in result.error
            yield Ok(None)

        run_effect(test_impl)
    finally:
        cleanup_api_files("test_error")


def test_ensure_directories_error(mocker):
    """Test error handling when creating directories fails"""
    try:
        # Use a regular function since ensure_directory is now a regular function
        def mock_ensure_directory(path: Path):
            return Error(str(FileError("Failed to create directory", str(path))))

        mocker.patch("fcship.commands.api.ensure_directory", mock_ensure_directory)

        @effect.result[None, str]()
        def test_impl():
            result = yield from create_api("test_dir_error")
            assert result.is_error()
            assert "Failed to create directory" in str(result.error)
            yield Ok(None)

        run_effect(test_impl)
    finally:
        cleanup_api_files("test_dir_error")


def test_create_files_error(mocker):
    """Test error handling when creating files fails"""
    try:

        @effect.result[FileCreationTracker, str]()
        def mock_create_single_file(tracker, path_content: tuple[Path, str]):
            yield Error(str(FileError("Failed to write file", str(path_content[0]))))

        mocker.patch("fcship.commands.api.create_single_file", mock_create_single_file)

        @effect.result[None, str]()
        def test_impl():
            result = yield from create_api("test_file_error")
            assert result.is_error()
            assert "Failed to write file" in str(result.error)
            yield Ok(None)

        run_effect(test_impl)
    finally:
        cleanup_api_files("test_file_error")


def test_notify_success_error(mocker):
    """Test error handling when success notification fails"""
    try:

        @effect.result[None, str]()
        def mock_success_message(ctx, msg):
            yield Error("Failed to show success message")

        mocker.patch("fcship.commands.api.success_message", mock_success_message)

        @effect.result[None, str]()
        def test_impl():
            result = yield from create_api("test_notify_error")
            assert result.is_error()
            assert "Failed to show success message" in result.error
            yield Ok(None)

        run_effect(test_impl)
    finally:
        cleanup_api_files("test_notify_error")


def test_api_unexpected_error(mocker):
    """Test handling of unexpected errors in API command"""
    try:

        def mock_validate_operation(*args):
            raise Exception("Unexpected error")

        mocker.patch("fcship.commands.api.validate_operation", mock_validate_operation)

        @effect.result[None, str]()
        def test_impl():
            result = yield from api("create", "test_error")
            assert result.is_error()
            assert "Unexpected error" in result.error
            yield Ok(None)

        run_effect(test_impl)
    finally:
        cleanup_api_files("test_error")


def test_create_api_unexpected_error(mocker):
    """Test handling of unexpected errors in create_api"""
    try:

        def mock_validate_name(*args):
            raise Exception("Unexpected validation error")

        mocker.patch("fcship.commands.api.validate_api_name", mock_validate_name)

        @effect.result[None, str]()
        def test_impl():
            result = yield from create_api("test_error")
            assert result.is_error()
            assert "Unexpected error" in result.error
            yield Ok(None)

        run_effect(test_impl)
    finally:
        cleanup_api_files("test_error")


def test_validate_api_name_invalid_identifier():
    """Test API name validation with invalid Python identifier"""
    try:

        @effect.result[None, str]()
        def test_impl():
            result = yield from validate_api_name("invalid-name")
            assert result.is_error()
            assert "must be a valid Python identifier" in result.error
            yield Ok(None)

        run_effect(test_impl)
    finally:
        cleanup_api_files("invalid-name")


def test_prepare_api_files_unexpected_error(mocker):
    """Test prepare_api_files with unexpected error"""
    try:

        def mock_get_templates(*args):
            raise Exception("Unexpected template error")

        mocker.patch("fcship.commands.api.get_api_templates", mock_get_templates)

        @effect.result[None, str]()
        def test_impl():
            result = yield from prepare_api_files("test_error")
            assert result.is_error()
            assert "Failed to prepare API files" in result.error
            assert "Unexpected template error" in result.error
            yield Ok(None)

        run_effect(test_impl)
    finally:
        cleanup_api_files("test_error")


def test_ensure_directories_unexpected_error(mocker):
    """Test ensure_api_directories with unexpected error"""
    try:
        # Use a regular function since ensure_directory is now a regular function
        def mock_ensure_directory(path: Path):
            return Error(str(FileError("Failed to create directory", str(path))))

        mocker.patch("fcship.commands.api.ensure_directory", mock_ensure_directory)

        @effect.result[None, str]()
        def test_impl():
            result = yield from ensure_api_directories()
            assert result.is_error()
            assert "Failed to create directory" in str(result.error)
            yield Ok(None)

        run_effect(test_impl)
    finally:
        cleanup_test_directories()


def test_create_files_unexpected_error(mocker):
    """Test create_api_files with unexpected error"""

    # Use a regular function for ensure_directory now
    def mock_ensure_directory(path: Path):
        return Ok(None)

    @effect.result[FileCreationTracker, str]()
    def mock_create_single_file(tracker, path_content: tuple[Path, str]):
        yield Error(str(FileError("Failed to write file", str(path_content[0]))))

    mocker.patch("fcship.commands.api.ensure_directory", mock_ensure_directory)
    mocker.patch("fcship.commands.api.create_single_file", mock_create_single_file)

    @effect.result[None, str]()
    def test_impl():
        files = Map.of_seq([("test.py", "content")])
        result = yield from create_api_files(ApiContext(name="test", files=files))
        assert result.is_error()
        assert "Failed to write file" in str(result.error)
        yield Ok(None)

    run_effect(test_impl)


def test_notify_success_unexpected_error(mocker):
    """Test notify_success with unexpected error"""

    def mock_success_message(*args):
        raise Exception("Unexpected notification error")

    mocker.patch("fcship.commands.api.success_message", mock_success_message)

    @effect.result[None, str]()
    def test_impl():
        result = yield from notify_success(
            ApiContext(name="test", files=Map()), FileCreationTracker()
        )
        assert result.is_error()
        assert "Failed to show success message" in result.error
        assert "Unexpected notification error" in result.error
        yield Ok(None)

    run_effect(test_impl)


def test_api_unexpected_error_in_create(mocker):
    """Test api function with unexpected error in create_api"""

    def mock_create_api(*args):
        raise Exception("Unexpected create error")

    mocker.patch("fcship.commands.api.create_api", mock_create_api)

    @effect.result[None, str]()
    def test_impl():
        result = yield from api("create", "test_error")
        assert result.is_error()
        assert "Unexpected error" in result.error
        assert "Unexpected create error" in result.error
        yield Ok(None)

    run_effect(test_impl)


def test_ensure_directories_general_error(mocker):
    """Test ensure_api_directories with a general exception"""
    try:
        # Use a regular function since ensure_directory is now a regular function
        def mock_ensure_directory(path: Path):
            if str(path) == "api/v1":  # Trigger error on specific directory
                return Error("General directory error")
            return Ok(None)

        mocker.patch("fcship.commands.api.ensure_directory", mock_ensure_directory)

        @effect.result[None, str]()
        def test_impl():
            result = yield from ensure_api_directories()
            assert result.is_error()
            assert "Failed to create API directories" in result.error
            assert "General directory error" in result.error
            yield Ok(None)

        run_effect(test_impl)
    finally:
        cleanup_test_directories()


def test_create_files_general_error(mocker):
    """Test create_api_files with a general exception"""
    try:
        # Use a regular function for ensure_directory now
        def mock_ensure_directory(path: Path):
            return Ok(None)

        @effect.result[FileCreationTracker, str]()
        def mock_create_single_file(tracker, path_content: tuple[Path, str]):
            raise RuntimeError("General file error")

        mocker.patch("fcship.commands.api.ensure_directory", mock_ensure_directory)
        mocker.patch("fcship.commands.api.create_single_file", mock_create_single_file)

        @effect.result[None, str]()
        def test_impl():
            files = Map.of_seq([("test.py", "content")])
            result = yield from create_api_files(ApiContext(name="test", files=files))
            assert result.is_error()
            assert "Failed to create API files" in result.error
            assert "General file error" in result.error
            yield Ok(None)

        run_effect(test_impl)
    finally:
        cleanup_api_files("test")


def test_api_general_error(mocker):
    """Test api function with a general exception"""
    try:

        @effect.result[tuple[str, str], str]()
        def mock_validate_operation(operation: str, name: str):
            yield Ok((operation, name))

        @effect.result[str, str]()
        def mock_create_api(name: str):
            raise RuntimeError("General API error")

        mocker.patch("fcship.commands.api.validate_operation", mock_validate_operation)
        mocker.patch("fcship.commands.api.create_api", mock_create_api)

        @effect.result[None, str]()
        def test_impl():
            result = yield from api("create", "test_error")
            assert result.is_error()
            assert "Unexpected error" in result.error
            assert "General API error" in result.error
            yield Ok(None)

        run_effect(test_impl)
    finally:
        cleanup_api_files("test_error")



"""API templates"""


def get_api_templates(name: str) -> dict[str, str]:
    """Get templates for API files."""
    return {
        f"api/v1/{name}.py": f"""from fastapi import APIRouter, Depends, HTTPException
from app.api.deps import get_current_user
from service.{name}.service import {name.title()}Service
from app.api.schemas.{name} import {name.title()}Response

router = APIRouter(prefix='/{name}s', tags=['{name.title()}s'])

@router.get('/', response_model=list[{name.title()}Response])
async def list_{name}s(
    service: {name.title()}Service = Depends(),
    current_user = Depends(get_current_user)
):
    return await service.list()""",
        f"api/schemas/{name}.py": f"""from pydantic import BaseModel

class {name.title()}Base(BaseModel):
    name: str

class {name.title()}Create({name.title()}Base):
    pass

class {name.title()}Response({name.title()}Base):
    id: int

    class Config:
        orm_mode = True""",
        f"tests/api/test_{name}.py": f"""import pytest
from httpx import AsyncClient

async def test_{name}_endpoints(
    client: AsyncClient,
    db_session
):
    # Arrange
    # TODO: Setup test data
    
    # Act
    response = await client.get('/{name}s/')
    
    # Assert
    assert response.status_code == 200
    assert isinstance(response.json(), list)""",
    }



"""Functional programming utilities."""

import asyncio
import functools
from collections.abc import Awaitable, Callable, Sequence
from typing import Any, ParamSpec, TypeVar

from expression import Error, Ok, Option, Result

A = TypeVar("A")
B = TypeVar("B")
C = TypeVar("C")
P = ParamSpec("P")


def lift_option(fn: Callable[P, Option[A]]) -> Callable[P, Result[A, Exception]]:
    """
    Eleva uma função que retorna um Option para uma função que retorna um Result.

    Se a função original retornar um Option com valor (Some), converte para Ok(valor).
    Caso contrário, retorna um Error com a mensagem "No value present".
    """
    return lambda *args, **kwargs: option_to_result(fn(*args, **kwargs), "No value present")


async def collect_results(
    results: Sequence[Awaitable[Result[A, Exception]]],
) -> Result[Sequence[A], Exception]:
    """Collect multiple async Results into a single Result containing all values.
    Short-circuits on first Error."""
    try:
        completed = await asyncio.gather(*results)
        return sequence_results(completed)
    except Exception as e:
        return Error(e)


def sequence_results(results: Sequence[Result[A, Exception]]) -> Result[Sequence[A], Exception]:
    """Convert a sequence of Results into a Result of sequence.
    Short-circuits on first Error."""
    from functools import reduce

    return reduce(
        lambda acc, r: acc.bind(lambda xs: r.map(lambda x: xs + [x])), results, Ok([])
    ).bind(lambda lst: Ok(lst))


def tap(fn: Callable[[A], Any]) -> Callable[[A], A]:
    """
    Cria uma função que executa um efeito colateral e retorna o mesmo valor inalterado.

    Útil para inserir chamadas de debug, log ou outras operações de side effect em cadeias
    de transformações puras.
    """

    @functools.wraps(fn)
    def tapped(value: A) -> A:
        fn(value)
        return value

    return tapped


def tap_async(fn: Callable[[A], Awaitable[Any]]) -> Callable[[A], Awaitable[A]]:
    """
    Cria uma função assíncrona que executa um efeito colateral e retorna o mesmo valor inalterado.

    Essa função é útil para incorporar operações assíncronas (como log assíncrono ou
    atualização de métricas) em cadeias de operações.
    """

    @functools.wraps(fn)
    def tapped(value: A) -> Awaitable[A]:
        async def inner() -> A:
            await fn(value)
            return value

        return inner()

    return tapped


def option_to_result(opt: Option[A], error_msg: str) -> Result[A, Exception]:
    """
    Converte uma instância de Option em um Result.

    Se o Option não contiver valor (ou seja, for None), retorna um Error com a mensagem
    de erro fornecida. Caso contenha valor, retorna um Ok contendo o valor.
    """
    return Error(ValueError(error_msg)) if opt.is_none() else Ok(opt.value)


def catch_errors(fn: Callable[P, A]) -> Callable[P, Result[A, Exception]]:
    """
    Wraps a function to catch any exceptions and return them as Result.Error.
    If the function succeeds, returns Result.Ok with the value.
    If the function already returns a Result, it is returned unchanged.
    """

    @functools.wraps(fn)
    def wrapped(*args: P.args, **kwargs: P.kwargs) -> Result[A, Exception]:
        try:
            result = fn(*args, **kwargs)
            if isinstance(result, Result):
                return result
            return Ok(result)
        except Exception as e:
            return Error(e)

    return wrapped



"""Service templates"""


def get_service_templates(name: str) -> dict[str, str]:
    """Get templates for service files."""
    return {
        "service.py": f"""from domain.{name}.repository import {name.title()}Repository
from .dto import {name.title()}DTO

class {name.title()}Service:
    def __init__(self, repository: {name.title()}Repository):
        self.repository = repository""",
        "dto.py": f"""from dataclasses import dataclass
from domain.{name}.entity import {name.title()}Entity

@dataclass
class {name.title()}DTO:
    @classmethod
    def from_entity(cls, entity: {name.title()}Entity) -> '{name.title()}DTO':
        return cls()""",
        "mapper.py": f"""from domain.{name}.entity import {name.title()}Entity
from .dto import {name.title()}DTO

def to_dto(entity: {name.title()}Entity) -> {name.title()}DTO:
    return {name.title()}DTO.from_entity(entity)""",
    }



import ast
from pathlib import Path

from .compact_code_visitor import CompactCodeVisitor
from .compactors import (
    get_compact_class_signature,
    get_compact_enum_signature,
    get_compact_function_signature,
    get_compact_method_signature,
)
from .file_utils import find_python_files


def read_compact_notation_guide(notation_file: str) -> str:
    """Lê o guia de notação compacta do arquivo."""
    try:
        with open(notation_file, encoding="utf-8") as f:
            return f.read()
    except Exception as e:
        print(f"Erro ao ler o guia de notação compacta: {e}")
        return "Guia de notação não disponível."


def get_files_to_process(project_root, target, ignore_dirs, ignore_files, verbose):
    """Determina quais arquivos processar com base no alvo especificado."""
    if target:
        target_path = Path(target)
        if target_path.is_file() and target_path.suffix == '.py':
            # Processar apenas um arquivo específico
            python_files = [target_path]
        elif target_path.is_dir():
            # Processar arquivos em um diretório específico
            python_files = find_python_files(target_path, ignore_dirs, ignore_files)
        else:
            print(f"Erro: O alvo {target} não é um arquivo Python válido ou diretório.")
            return []
    else:
        # Comportamento padrão: encontrar todos os arquivos Python no projeto
        python_files = find_python_files(Path(project_root), ignore_dirs, ignore_files)
    
    if verbose:
        print(f"Encontrados {len(python_files)} arquivos Python para processar")
    
    return python_files


def process_files(python_files, verbose):
    """Processa os arquivos Python e gera as representações compactas."""
    all_compact_lines = []
    total_classes = 0
    total_functions = 0
    total_methods = 0
    total_enums = 0

    for file_path in python_files:
        try:
            with open(file_path, encoding="utf-8") as f:
                file_content = f.read()
            
            # Parse do arquivo e processamento dos nós
            file_ast = ast.parse(file_content)
            visitor = CompactCodeVisitor()
            visitor.visit(file_ast)

            # Adicionar cabeçalho do arquivo
            compact_lines = [f"file:{file_path}"]
            
            # Processar importações (opcional) - apenas indicar o intervalo de linhas
            if visitor.imports and visitor.import_lines:
                min_line = min(visitor.import_lines)
                max_line = max(visitor.import_lines)
                imports_line = f"imports:ln{min_line}:{max_line}"
                compact_lines.append(imports_line)

            # Criar um dicionário para mapear classes para seus métodos e enums
            class_elements = {}
            
            # Inicializar entradas para todas as classes
            for class_node in visitor.classes:
                class_elements[class_node] = {
                    'methods': [],
                    'enums': []
                }
            
            # Agrupar métodos com suas classes
            for class_node, method_node in visitor.methods:
                if class_node in class_elements:
                    class_elements[class_node]['methods'].append(method_node)
                    total_methods += 1
            
            # Agrupar enums com suas classes
            for class_node, enum_node in visitor.enums:
                if class_node in class_elements:
                    class_elements[class_node]['enums'].append(enum_node)
                    total_enums += 1
            
            # Processar classes com seus elementos aninhados
            for class_node in visitor.classes:
                is_dataclass = any(
                    isinstance(dec, ast.Name) and dec.id == 'dataclass'
                    for dec in class_node.decorator_list
                )
                class_signature = get_compact_class_signature(class_node, is_dataclass)
                
                # Obter todos os métodos e enums desta classe
                methods = class_elements[class_node]['methods']
                enums = class_elements[class_node]['enums']
                
                # Se não houver elementos aninhados, apenas adicione a classe
                if not methods and not enums:
                    # Obter informações de linha
                    start_line = class_node.lineno
                    end_line = max([node.lineno for node in class_node.body] + [start_line]) if class_node.body else start_line
                    line_info = f";ln{start_line}:{end_line}"
                    
                    compact_lines.append(f"{class_signature}{line_info}")
                else:
                    # Separar métodos dunder e métodos regulares
                    dunder_methods = [m for m in methods if m.name.startswith('__') and m.name.endswith('__')]
                    regular_methods = [m for m in methods if not (m.name.startswith('__') and m.name.endswith('__'))]
                    
                    # Gerar representações compactas para cada tipo
                    dunder_signatures = [get_compact_method_signature(method) for method in dunder_methods]
                    method_signatures = [get_compact_method_signature(method) for method in regular_methods]
                    enum_signatures = [get_compact_enum_signature(enum) for enum in enums]
                    
                    # Criar grupos de elementos
                    nested_groups = []
                    if dunder_signatures:
                        nested_groups.append(f"{','.join(dunder_signatures)}")
                    if method_signatures:
                        nested_groups.append(f"{','.join(method_signatures)}")
                    if enum_signatures:
                        nested_groups.append(f"{','.join(enum_signatures)}")
                    
                    # Obter informações de linha
                    start_line = class_node.lineno
                    all_lines = [class_node.lineno]
                    for method in methods:
                        all_lines.append(method.lineno)
                        all_lines.extend([node.lineno for node in method.body if hasattr(node, 'lineno')])
                    for enum in enums:
                        all_lines.append(enum.lineno)
                    end_line = max(all_lines) if all_lines else start_line
                    line_info = f";ln{start_line}:{end_line}"
                    
                    # Combinar tudo em uma única linha
                    compact_line = f"{class_signature}[{'|'.join(nested_groups)}]{line_info}"
                    compact_lines.append(compact_line)
                
                total_classes += 1

            # Processar funções de nível superior
            for func_node in visitor.functions:
                # Obter informações de linha
                start_line = func_node.lineno
                end_line = max([node.lineno for node in func_node.body] + [start_line]) if func_node.body else start_line
                line_info = f";ln{start_line}:{end_line}"
                
                compact_lines.append(f"{get_compact_function_signature(func_node)}{line_info}")
                total_functions += 1
            
            # Adicionar separador ao final do arquivo em vez de linha vazia
            compact_lines.append("-")

            all_compact_lines.extend(compact_lines)

        except Exception as e:
            print(f"Erro ao processar {file_path}: {e}")
            all_compact_lines.append(f"# Erro ao processar {file_path}: {e}")

    return all_compact_lines, {
        'total_classes': total_classes,
        'total_functions': total_functions,
        'total_methods': total_methods,
        'total_enums': total_enums
    }


def write_output(all_compact_lines, output_file, stdout, verbose, stats):
    """Escreve o resultado no arquivo ou envia para o console."""
    # Encontrar o índice da linha separadora
    separator_index = -1
    for i, line in enumerate(all_compact_lines):
        if line.startswith("-" * 10):  # Linha separadora começa com pelo menos 10 hífens
            separator_index = i
            break
    
    # Se não encontrou o separador, aplicar replaces a tudo
    if separator_index == -1:
        clean_lines = []
        for line in all_compact_lines:
            # Preservar espaços em linhas de comentário
            if line.startswith('#'):
                clean_lines.append(line)
            else:
                clean_lines.append(line.replace("__", "").replace(" ", ""))
    else:
        # Manter o guia original (com espaços) e modificar apenas o conteúdo compactado
        guide_lines = all_compact_lines[:separator_index+1]  # Incluir o separador
        content_lines = []
        for line in all_compact_lines[separator_index+1:]:
            if line.startswith('#'):
                content_lines.append(line)
            else:
                content_lines.append(line.replace("__", "").replace(" ", ""))
        clean_lines = guide_lines + content_lines
    
    if stdout:
        # Enviar para o console (stdout)
        print("\n".join(clean_lines))
    else:
        # Escrever no arquivo
        with open(output_file, "w", encoding="utf-8") as f:
            f.write("\n".join(clean_lines))
        if verbose:
            print(f"\nGerado com sucesso: {output_file}")

    # Exibir estatísticas
    print("\nEstatísticas:")
    print(f"- Classes/Dataclasses: {stats['total_classes']}")
    print(f"- Funções: {stats['total_functions']}")
    print(f"- Métodos: {stats['total_methods']}")
    print(f"- Enums/Constantes: {stats['total_enums']}")
    print(f"- Total de elementos: {stats['total_classes'] + stats['total_functions'] + stats['total_methods'] + stats['total_enums']}")


def generate_compact_code_with_config(
    output_file,
    project_root,
    notation_file,
    ignore_dirs,
    ignore_files,
    include_guide=True,
    verbose=False,
    target=None,
    stdout=False
):
    """
    Gera o código compacto com base na configuração.
    
    Args:
        output_file: Caminho para o arquivo de saída
        project_root: Diretório raiz do projeto
        notation_file: Caminho para o arquivo de guia de notação
        ignore_dirs: Lista de padrões de diretórios a serem ignorados
        ignore_files: Lista de padrões de arquivos a serem ignorados
        include_guide: Se deve incluir o guia de notação no início
        verbose: Se deve exibir informações detalhadas
        target: Arquivo ou diretório específico a ser processado
        stdout: Se deve enviar a saída para stdout em vez de um arquivo
        
    Returns:
        str: Caminho para o arquivo de saída (None se stdout=True)
    """
    
    # Sempre incluir o guia de notação
    guide_content = read_compact_notation_guide(notation_file)
    guide_lines = guide_content.splitlines()
    
    # Obter os arquivos Python para processar
    python_files = get_files_to_process(project_root, target, ignore_dirs, ignore_files, verbose)
    
    # Processar os arquivos e gerar o conteúdo compacto
    compact_lines, stats = process_files(python_files, verbose)
    
    # Combinar o guia e o conteúdo compacto
    all_compact_lines = []
    
    # Adicionar o guia de notação
    all_compact_lines.extend(guide_lines)
    
    # Adicionar uma linha separadora simples entre o guia e o conteúdo
    all_compact_lines.append("-" * 80)
    
    # Adicionar o conteúdo compacto
    all_compact_lines.extend(compact_lines)
    
    # Escrever o resultado
    write_output(all_compact_lines, output_file, stdout, verbose, stats)
    
    # Retornar o caminho do arquivo se não estiver usando stdout
    return None if stdout else output_file


if __name__ == "__main__":
    generate_compact_code_with_config(
        output_file="compact_code.txt",
        project_root=".",
        notation_file="notation_guide.txt",
        ignore_dirs=["venv", ".git"],
        ignore_files=["__init__.py"],
        include_guide=True,
        verbose=True,
        target=None,
        stdout=False
    )



"""Entry point for the compact command."""

from typing import Literal

from expression import Error, Ok, Result

from .cli import parse_arguments
from .config import (
    COMPACT_NOTATION_FILE,
    DEFAULT_OUTPUT_FILE,
    IGNORE_DIRS,
    IGNORE_FILES,
)
from .generator import generate_compact_code_with_config
from .token_counter import analyze_file, print_token_analysis


def compact(
    output_file: str | None = None,
    project_root: str = ".",
    notation_file: str | None = None,
    ignore_dirs: list[str] | None = None,
    ignore_files: list[str] | None = None,
    target: str | None = None, 
    stdout: bool = False,
    verbose: bool = False,
    count_tokens: bool = False,
    token_model: Literal["gpt-4o", "gpt-3.5-turbo", "claude-3-opus"] = "gpt-4o",
) -> Result[str, str]:
    """Generate compact code representation of Python files.
    
    Args:
        output_file: Path to output file (default: compact_code.md)
        project_root: Project root directory (default: current directory)
        notation_file: Path to compact notation guide file
        ignore_dirs: List of directory patterns to ignore
        ignore_files: List of file patterns to ignore
        target: Target file or directory to process
        stdout: Print output to stdout instead of file
        verbose: Enable verbose output
        count_tokens: Count tokens in the output file
        token_model: Model to use for token counting
        
    Returns:
        Result containing path to the output file or error message
    """
    try:
        # Use provided values or defaults from config
        final_output_file = output_file or DEFAULT_OUTPUT_FILE
        final_notation_file = notation_file or COMPACT_NOTATION_FILE
        
        # Process ignore patterns
        final_ignore_dirs = IGNORE_DIRS.copy()
        final_ignore_files = IGNORE_FILES.copy()
        
        if ignore_dirs:
            final_ignore_dirs.extend(ignore_dirs)
        if ignore_files:
            final_ignore_files.extend(ignore_files)
            
        # Generate compact code
        result_file = generate_compact_code_with_config(
            output_file=final_output_file,
            project_root=project_root,
            notation_file=final_notation_file,
            ignore_dirs=final_ignore_dirs,
            ignore_files=final_ignore_files,
            target=target,
            stdout=stdout,
            verbose=verbose
        )
        
        # Count tokens if requested and output file was generated
        if count_tokens and result_file and not stdout:
            stats = analyze_file(result_file, token_model)
            print_token_analysis(stats)
            
        if stdout:
            return Ok("Compact code generated and printed to stdout")
        return Ok(f"Compact code generated at {result_file}")
            
    except Exception as e:
        return Error(f"Error generating compact code: {e}")

def compact_command() -> Result[str, str]:
    """Command-line entry point for the compact command.
    
    Returns:
        Result containing the output message or error
    """
    try:
        args = parse_arguments()
        
        # Process ignore patterns
        ignore_dirs = IGNORE_DIRS.copy()
        ignore_files = IGNORE_FILES.copy()
        
        if args.ignore_dirs:
            ignore_dirs.extend(args.ignore_dirs.split(","))
        if args.ignore_files:
            ignore_files.extend(args.ignore_files.split(","))
        
        return compact(
            output_file=args.output,
            project_root=args.directory,
            notation_file=args.guide,
            ignore_dirs=ignore_dirs,
            ignore_files=ignore_files,
            target=args.target,
            stdout=args.stdout,
            verbose=args.verbose,
            count_tokens=args.count_tokens,
            token_model=args.token_model
        )
            
    except Exception as e:
        return Error(f"Error in compact command: {e}") 